@@ -1,20 +1,26 @@
-"""
-Definition List Extension for Python-Markdown
-=============================================
+# Definition List Extension for Python-Markdown
+# =============================================
 
+# Adds parsing of Definition Lists to Python-Markdown.
+
+# See https://Python-Markdown.github.io/extensions/definition_lists
+# for documentation.
+
+# Original code Copyright 2008 [Waylan Limberg](http://achinghead.com)
+
+# All changes Copyright 2008-2014 The Python Markdown Project
+
+# License: [BSD](https://opensource.org/licenses/bsd-license.php)
+
+"""
 Adds parsing of Definition Lists to Python-Markdown.
 
-See <https://Python-Markdown.github.io/extensions/definition_lists>
-for documentation.
-
-Original code Copyright 2008 [Waylan Limberg](http://achinghead.com)
-
-All changes Copyright 2008-2014 The Python Markdown Project
-
-License: [BSD](https://opensource.org/licenses/bsd-license.php)
-
+See the [documentation](https://Python-Markdown.github.io/extensions/definition_lists)
+for details.
 """
 
+from __future__ import annotations
+
 from . import Extension
 from ..blockprocessors import BlockProcessor, ListIndentProcessor
 import xml.etree.ElementTree as etree
@@ -27,10 +33,10 @@ class DefListProcessor(BlockProcessor):
     RE = re.compile(r'(^|\n)[ ]{0,3}:[ ]{1,3}(.*?)(\n|$)')
     NO_INDENT_RE = re.compile(r'^[ ]{0,3}[^ :]')
 
-    def test(self, parent, block):
+    def test(self, parent: etree.Element, block: str) -> bool:
         return bool(self.RE.search(block))
 
-    def run(self, parent, blocks):
+    def run(self, parent: etree.Element, blocks: list[str]) -> bool | None:
 
         raw_block = blocks.pop(0)
         m = self.RE.search(raw_block)
@@ -89,9 +95,11 @@ class DefListIndentProcessor(ListIndentProcessor):
 
     # Definition lists need to be aware of all list types
     ITEM_TYPES = ['dd', 'li']
+    """ Include `dd` in list item types. """
     LIST_TYPES = ['dl', 'ol', 'ul']
+    """ Include `dl` is list types. """
 
-    def create_item(self, parent, block):
+    def create_item(self, parent: etree.Element, block: str) -> None:
         """ Create a new `dd` or `li` (depending on parent) and parse the block with it as the parent. """
 
         dd = etree.SubElement(parent, 'dd')
